package Fairness.Nodes.TreeBehaviors;

import Fairness.Nodes.Node;

/**
 * TreeBehavior is an interface and is a part of the Taxation scheme which we are not implementing.
 *
 * @author Jason Winnebeck
 */
public interface TreeBehavior {
    /**
     * Returns true if the Node wants to join more stripes.
     *
     * @param myNode non-null Node to which this behavior belongs.
     *
     * @return true if the Node wants to join more stripes.
     */
    public boolean wantJoinMore( Node myNode );

    /**
     * Tells the node to join another stripe.
     *
     * @param myNode non-null Node to which this behavior belongs.
     *
     * @throws IllegalStateException if {@link #wantJoinMore(Fairness.Nodes.Node)} returns false.
     */
    public void joinNextStripe( Node myNode );

    /**
     * Tells this behavior to remove the Node from a stripe, to reduce its subscription level by
     * one.
     *
     * @param myNode non-null Node to which this behavior belongs.
     */
    public void partStripe( Node myNode );
}
