package Fairness.Nodes;

import org.gillius.Util.CheckUtils;

import java.util.HashSet;
import java.util.Set;

/**
 * TreeInfo contains information on a {@link Node}'s participation in a single stripe tree.
 *
 * @author Jason Winnebeck
 */
public class TreeInfo {
    private final int stripeId;

    private Node parent = null;
    private final Set<Node> children = new HashSet<Node>();

    /**
     * Constructs a new TreeInfo
     *
     * @param stripeId stripe ID this information is for
     */
    public TreeInfo( int stripeId ) {
        this.stripeId = stripeId;
    }

    public int getStripeId() {
        return stripeId;
    }

    /**
     * Returns the parent of the node in this stripe. Null means that a parent has not been assigned
     * yet (if during tree rebuild), or that the publisher is the parent (after tree rebuild).
     *
     * @return nullable parent
     */
    public Node getParent() {
        return parent;
    }

    /**
     * Adds the given child to this node for this TreeInfo's stripe.
     *
     * @param child non-null Node
     */
    public void addChild( Node child ) {
        CheckUtils.checkNull( child, "child" );

        children.add( child );
    }

    /**
     * Removes the given child from this node for this TreeInfo's stripe, if the given node is
     * actually a child.
     *
     * @param child non-null Node
     */
    public void removeChild( Node child ) {
        CheckUtils.checkNull( child, "child" );

        children.remove( child );
    }

    /**
     * Sets the Node's parent.
     */
    public void setParent( Node parent ) {
        this.parent = parent;
    }

    /**
     * Return a set of all current children of the node for this stripe.
     */
    public Set<Node> getChildren() {
        return children;
    }
}
