package Fairness;

import Fairness.Trees.TreeManager;
import Fairness.Nodes.Node;
import Fairness.Nodes.NodePacket;

/**
 * PublisherSendEvent implements the event when the publisher sends a packet on each stripe to the
 * roots of each stripe.
 *
 * @author Jason Winnebeck
 */
public class PublisherSendEvent implements SimulationEvent {
    private final TreeManager manager;

    /**
     * Constructs a new PublisherSendEvent with the TreeManager that is needed to get all of the
     * root nodes for each stripe.
     */
    public PublisherSendEvent( TreeManager manager ) {
        this.manager = manager;
    }

    /**
     * Performs the event by sending a packet to each stripe root Node.
     *
     * @param sim  non-null Simulation
     * @param time simulation time, exactly equal to the result of {@link Simulation#getTime()}
     */
    public void perform( Simulation sim, double time ) {
        int packetSize = sim.getParams().getPacketSize();
        for ( Node root : manager.getRootNodes() ) {
            NodePacket packet = new NodePacket( null, root.getStripeId(), false, true, packetSize );
            root.processPacket( sim, packet );
        }

        //Schedule this event to occur again
        double nextTime = time + (double) ( packetSize * 8 ) / sim.getParams().getStripeBps();
        sim.scheduleEvent( nextTime, this );
    }
}
