package Fairness.Trees;

import Fairness.NodeMonitor;
import Fairness.Nodes.Node;
import Fairness.Nodes.NodeBehaviors.SimpleNodeBehavior;
import Fairness.Nodes.TreeBehaviors.TreeBehavior;
import Fairness.Simulation;
import Fairness.SimulationParameters;

/**
 * NewRandomTest is used for testing the RandomTreeManager.
 *
 * @author John Kaeuper
 */
public class NewRandomTest {

    public static void main( String[] args ) {

        SimulationParameters params =
            new SimulationParameters( 4, 51200, 512, 50.0, 50 );

        Simulation sim = new Simulation( params );

        TreeManager manager = new RandomTreeManager();
        manager.setUp( sim );
        SimpleNodeBehavior nodeBehavior = new SimpleNodeBehavior();
        NoTreeBehavior treeBehavior = new NoTreeBehavior();
        NodeMonitor monitor = new NodeMonitor();

        // make 20 Nodes and give them to manager; each Node can handle
        // 2 children and can receive all 16 stripes
        for ( int i = 0; i < 10; ++i ) {
            int aNodeId = manager.getNextNodeId();
            Node aNode = new Node( sim, aNodeId, 204800, 204800, monitor,
                                   nodeBehavior, treeBehavior );
            manager.addNode( aNode );
        }

        // now build the forest
        manager.rebuildTree();

        // now let's see what the stripe trees look like.....
        System.out.println( manager.forestToString() );


    } // main

    private static class NoTreeBehavior implements TreeBehavior {
        public boolean wantJoinMore( Node myNode ) {
            return false;
        }

        public void joinNextStripe( Node myNode ) {
        }

        public void partStripe( Node myNode ) {
        }
    }

} // PastryTreeTest
