package Fairness.Trees;

import Fairness.NodeMonitor;
import Fairness.Nodes.Node;
import Fairness.Nodes.NodeBehaviors.SimpleNodeBehavior;
import Fairness.Nodes.TreeBehaviors.TreeBehavior;
import Fairness.Simulation;
import Fairness.SimulationParameters;

/**
 * PastryTreeTest is used for testing and debugging purpose and it contructs nodes by hardcoding the values.
 * It calls the rebuild method for building the forest trees.
 *
 * @author John Kaeuper
 */
public class PastryTreeTest {
    public static void main( String[] args ) {

        SimulationParameters params =
            new SimulationParameters( 16, 51200, 512, 50.0, 50 );

        Simulation sim = new Simulation( params );

        PastryTreeManager manager = new PastryTreeManager();
        manager.setUp( sim );
        SimpleNodeBehavior nodeBehavior = new SimpleNodeBehavior();
        NoTreeBehavior treeBehavior = new NoTreeBehavior();
        NodeMonitor monitor = new NodeMonitor();

        // make 20 Nodes and give them to manager; each Node can handle
        // 2 children and can receive all 16 stripes
        for ( int i = 0; i < 20; ++i ) {
            int aNodeId = manager.getNextNodeId();
            Node aNode = new Node( sim, aNodeId, 102400, 819200, monitor,
                                   nodeBehavior, treeBehavior );
            manager.addNode( aNode );
        }

        // now build the forest
        manager.rebuildTree();

        // now let's see what the stripe trees look like.....
        System.out.println( manager.forestToString() );


    } // main

    private static class NoTreeBehavior implements TreeBehavior {
        public boolean wantJoinMore( Node myNode ) {
            return false;
        }

        public void joinNextStripe( Node myNode ) {
        }

        public void partStripe( Node myNode ) {
        }
    }

} // PastryTreeTest
