package Fairness;

import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;

/**
 * SimulationTest
 *
 * @author Jason Winnebeck
 */
public class SimulationTest {
	private Simulation sim;

	@Before public void setUp() {
		sim = new Simulation( new SimulationParameters( 1, 1024, 512, 100.0, 0 ) );
	}

	@Test public void startTime() {
		assertEquals( 0.0, sim.getTime(), 0.0 );
	}

	@Test(expected=IllegalArgumentException.class)
	public void BadScheduling() {
		sim.scheduleEvent( -1.0, new DummyEvent() );
	}

	@Test(expected=IllegalArgumentException.class)
	public void BadScheduling2() {
		sim.scheduleEvent( 1.0, new DummyEvent() );
		sim.scheduleEvent( 2.0, new DummyEvent() );
		sim.scheduleEvent( 3.0, new DummyEvent() );

		sim.next();
		sim.next();

		sim.scheduleEvent( 1.0, new DummyEvent() );
	}

	@Test public void testEvents() {
		Counter c = new Counter();
		sim.scheduleEvent( 15.0, new EventVerifier( 15.0, sim, c, 0 ) );
		sim.scheduleEvent( 25.0, new EventVerifier( 25.0, sim, c, 1 ) );
		sim.scheduleEvent( 35.0, new EventVerifier( 35.0, sim, c, 2 ) );
		sim.scheduleEvent( 45.0, new EventVerifier( 45.0, sim, c, 3 ) );

		assertTrue( sim.next() );
		assertTrue( sim.next() );
		assertTrue( sim.next() );
		assertFalse( sim.next() );
	}

	@Test public void testEvents2() {
		Counter c = new Counter();
		sim.scheduleEvent( 25.0, new EventVerifier( 25.0, sim, c, 1 ) );
		sim.scheduleEvent( 15.0, new EventVerifier( 15.0, sim, c, 0 ) );
		sim.scheduleEvent( 45.0, new EventVerifier( 45.0, sim, c, 3 ) );
		sim.scheduleEvent( 35.0, new EventVerifier( 35.0, sim, c, 2 ) );

		assertTrue( sim.next() );
		assertTrue( sim.next() );
		assertTrue( sim.next() );
		assertFalse( sim.next() );
	}

	@Test public void testEvents3() {
		Counter c = new Counter();
		sim.scheduleEvent( 25.0, new EventVerifier( 25.0, sim, c, 1 ) );
		sim.scheduleEvent( 15.0, new EventVerifier( 15.0, sim, c, 0 ) );
		sim.scheduleEvent( 45.0, new EventVerifier( 45.0, sim, c, 4 ) );
		sim.scheduleEvent( 35.0, new EventVerifier( 35.0, sim, c, 2 ) );

		assertTrue( sim.next() );
		assertTrue( sim.next() );
		assertTrue( sim.next() );

		sim.scheduleEvent( 35.1, new EventVerifier( 35.1, sim, c, 3 ) );

		assertTrue( sim.next() );
		assertFalse( sim.next() );
	}

	private static class DummyEvent implements SimulationEvent {
		public void perform( Simulation sim, double time ) {
		}
	}

	private static class Counter {
		public int count = 0;
	}

	private static class EventVerifier implements SimulationEvent {
		private final double expectedTime;
		private final Simulation expectedSim;
		private final Counter count;
		private final int expectedCount;

		public EventVerifier( double expectedTime, Simulation expectedSim, Counter count,
		                      int expectedCount ) {
			this.expectedTime = expectedTime;
			this.expectedSim = expectedSim;
			this.count = count;
			this.expectedCount = expectedCount;
		}

		public void perform( Simulation sim, double time ) {
			assertSame( expectedSim, sim );
			assertEquals( expectedTime, time, 0.0 );
			assertEquals( expectedCount, count.count );
			count.count++;
		}
	}
}
